==> pgrpm-0.1.0.tar.bz2: pgrpm-0.1.0/ds.c <==
/* $Id */
#include <postgres.h>
#include <fmgr.h>

#include <rpm/rpmlib.h>
#include <rpm/rpmds.h>

#include "ds.h"

int _overlap_evr(int fla, char * evra,
                 int flb, char * evrb)
{
    int rc = 0;
    rpmds dsa = NULL;
    rpmds dsb = NULL;
    
    if (!evra || !evrb) return 0;
    dsa = rpmdsSingle(RPMTAG_PROVIDENAME, "", evra, fla);
    dsb = rpmdsSingle(RPMTAG_PROVIDENAME, "", evrb, flb);

    (void) rpmdsSetIx(dsa, 0);
    (void) rpmdsSetIx(dsb, 0);

    rc = rpmdsCompare(dsa, dsb);

    dsa = rpmdsFree(dsa);
    dsb = rpmdsFree(dsb);

    return rc;
}

int _pg_overlap_nevr(text * tna, int fla, text * tevra,
                     text * tnb, int flb, text * tevrb)
{
    int rc = 0;
    char * evra = NULL;
    char * evrb = NULL;

    if (tna && tnb) {
        if(VARSIZE(tna) != VARSIZE(tnb))
            return 0;
        if(memcmp(VARDATA(tna), VARDATA(tnb), VARSIZE(tna) - VARHDRSZ))
            return 0;
    } else if (!tna && !tnb) {
    } else {
        return 0;
    }
    
    evra = GET_STR(tevra);
    evrb = GET_STR(tevrb);
    rc = _overlap_evr(fla, evra, flb, evrb);

#ifdef FREE_OURSELF 
    if(evra)
        pfree(evra);
    if(evrb)
        pfree(evrb);
#endif

    return rc;
}

PG_FUNCTION_INFO_V1(rpmdepmatch_nevr);
Datum
rpmdepmatch_nevr(PG_FUNCTION_ARGS)
{
    if(_pg_overlap_nevr(
        PG_ARGISNULL(0) ? NULL : PG_GETARG_TEXT_P(0),
        PG_ARGISNULL(1) ? 0 : PG_GETARG_INT32(1),
        PG_ARGISNULL(2) ? NULL : PG_GETARG_TEXT_P(2),
        PG_ARGISNULL(3) ? NULL : PG_GETARG_TEXT_P(3),
        PG_ARGISNULL(4) ? 0 : PG_GETARG_INT32(4),
        PG_ARGISNULL(5) ? NULL : PG_GETARG_TEXT_P(5)
        ))
        PG_RETURN_BOOL(TRUE);
    else
        PG_RETURN_BOOL(FALSE);
}

PG_FUNCTION_INFO_V1(rpmdepmatch_evr);
Datum
rpmdepmatch_evr(PG_FUNCTION_ARGS)
{
    if(_pg_overlap_nevr(
        NULL,
        PG_ARGISNULL(0) ? 0 : PG_GETARG_INT32(0),
        PG_ARGISNULL(1) ? NULL : PG_GETARG_TEXT_P(1),
        NULL,
        PG_ARGISNULL(2) ? 0 : PG_GETARG_INT32(2),
        PG_ARGISNULL(3) ? NULL : PG_GETARG_TEXT_P(3)
        ))
        PG_RETURN_BOOL(TRUE);
    else
        PG_RETURN_BOOL(FALSE);
}

==> pgrpm-0.1.4.tar.bz2: pgrpm-0.1.4/ds.c <==
/* $Id: ds.c,v 1.7 2005/05/16 00:22:19 nanardon Exp $ */

#include <postgres.h>
#include <fmgr.h>

#include <rpm/rpmlib.h>
#include <rpm/rpmds.h>

#include "ds.h"

int _overlap_evr(int fla, char * evra,
                 int flb, char * evrb)
{
    int rc = 0;
    rpmds dsa = NULL;
    rpmds dsb = NULL;
    
    dsa = rpmdsSingle(RPMTAG_PROVIDENAME, "", evra ? evra : "", fla);
    dsb = rpmdsSingle(RPMTAG_PROVIDENAME, "", evrb ? evrb : "", flb);

    (void) rpmdsSetIx(dsa, 0);
    (void) rpmdsSetIx(dsb, 0);

    rc = rpmdsCompare(dsa, dsb);

    dsa = rpmdsFree(dsa);
    dsb = rpmdsFree(dsb);

    return rc;
}

int _pg_overlap_nevr(text * tna, int fla, text * tevra,
                     text * tnb, int flb, text * tevrb)
{
    int rc = 0;
    char * evra = NULL;
    char * evrb = NULL;

    if (tna && tnb) {
        if(VARSIZE(tna) != VARSIZE(tnb))
            return 0;
        if(memcmp(VARDATA(tna), VARDATA(tnb), VARSIZE(tna) - VARHDRSZ))
            return 0;
    } else if (!tna && !tnb) {
    } else {
        return 0;
    }
    
    evra = GET_STR(tevra);
    evrb = GET_STR(tevrb);
    rc = _overlap_evr(fla, evra, flb, evrb);

#ifdef FREE_OURSELF 
    if(evra)
        pfree(evra);
    if(evrb)
        pfree(evrb);
#endif

    return rc;
}

PG_FUNCTION_INFO_V1(rpmdepmatch_nevr);
Datum
rpmdepmatch_nevr(PG_FUNCTION_ARGS)
{
    PG_RETURN_BOOL(_pg_overlap_nevr(
        PG_ARGISNULL(0) ? NULL : PG_GETARG_TEXT_P(0),
        PG_ARGISNULL(1) ? 0 : PG_GETARG_INT32(1),
        PG_ARGISNULL(2) ? NULL : PG_GETARG_TEXT_P(2),
        PG_ARGISNULL(3) ? NULL : PG_GETARG_TEXT_P(3),
        PG_ARGISNULL(4) ? 0 : PG_GETARG_INT32(4),
        PG_ARGISNULL(5) ? NULL : PG_GETARG_TEXT_P(5)
        ));
}

PG_FUNCTION_INFO_V1(rpmdepmatch_evr);
Datum
rpmdepmatch_evr(PG_FUNCTION_ARGS)
{
    PG_RETURN_BOOL(_pg_overlap_nevr(
        NULL,
        PG_ARGISNULL(0) ? 0 : PG_GETARG_INT32(0),
        PG_ARGISNULL(1) ? NULL : PG_GETARG_TEXT_P(1),
        NULL,
        PG_ARGISNULL(2) ? 0 : PG_GETARG_INT32(2),
        PG_ARGISNULL(3) ? NULL : PG_GETARG_TEXT_P(3)
        ));
}
